@@ -1,11 +1,13 @@
 """The root Python package for ScalarStop."""
 from scalarstop import (
     datablob,
+    datablob_metadata,
     dataclasses,
     hyperparams,
     model,
     model_template,
     pickle,
+    tf_config,
     train_store,
 )
 
@@ -16,6 +18,8 @@ from scalarstop import (
 from scalarstop.datablob import DataBlob  # isort:skip
 from scalarstop.datablob import DataFrameDataBlob  # isort:skip
 from scalarstop.datablob import AppendDataBlob  # isort:skip
+from scalarstop.datablob import DistributedDataBlob  # isort: skip
+from scalarstop.datablob_metadata import DataBlobMetadata  # isort:skip
 from scalarstop.dataclasses import dataclass  # isort:skip
 from scalarstop.model_template import ModelTemplate  # isort:skip
 from scalarstop.model import Model  # isort:skip
@@ -25,3 +29,4 @@ from scalarstop.hyperparams import HyperparamsType  # isort:skip
 from scalarstop.hyperparams import AppendHyperparamsType  # isort:skip
 from scalarstop.hyperparams import NestedHyperparamsType  # isort:skip
 from scalarstop.hyperparams import enforce_dict  # isort:skip
+from scalarstop.tf_config import TFConfig  # isort:skip
