@@ -12,9 +12,8 @@ that makes it easy to keep track of many datasets at once.
 """
 
 import errno
-import json
 import os
-from typing import Any, Dict, Mapping, Optional, Type, Union
+from typing import Any, Mapping, Optional, Type, Union
 
 import pandas as pd
 import tensorflow as tf
@@ -24,8 +23,6 @@ import scalarstop.pickle
 from scalarstop._constants import (
     _DATAFRAME_FILENAME,
     _DEFAULT_SAVE_LOAD_VERSION,
-    _METADATA_JSON_FILENAME,
-    _METADATA_PICKLE_FILENAME,
     _SUBTYPE_TEST,
     _SUBTYPE_TRAINING,
     _SUBTYPE_VALIDATION,
@@ -36,7 +33,7 @@ from scalarstop._logging import Timeblock
 from scalarstop._naming import temporary_filename
 from scalarstop._single_namespace import SingleNamespace
 from scalarstop._tfdata import tfdata_load, tfdata_save
-from scalarstop.dataclasses import asdict
+from scalarstop.datablob_metadata import DataBlobMetadata
 from scalarstop.exceptions import (
     DataBlobNotFound,
     ElementSpecNotFound,
@@ -51,13 +48,60 @@ from scalarstop.hyperparams import (
     HyperparamsType,
     NestedHyperparamsType,
     enforce_dict,
-    flatten_hyperparams,
 )
 
 _LOGGER = Logger(__name__)
 
 
-class DataBlob(SingleNamespace):
+class DataBlobBase(SingleNamespace):
+    """The abstract base class describing the properties common to all DataBlobs."""
+
+    def __init__(
+        self,
+        *,
+        hyperparams: Optional[Union[Mapping[str, Any], HyperparamsType]] = None,
+        **kwargs,
+    ):
+        super().__init__(hyperparams=hyperparams, **kwargs)
+        self._training: Any = None
+        self._validation: Any = None
+        self._test: Any = None
+
+    def set_training(self) -> Any:
+        """Creates and returns a new object representing the training set."""
+        raise IsNotImplemented("DataBlobBase.set_training()")
+
+    @property
+    def training(self) -> Any:
+        """An object representing the training set."""
+        if self._training is None:
+            self._training = self.set_training()
+        return self._training
+
+    def set_validation(self) -> Any:
+        """Creates and returns a new object representing the validation set."""
+        raise IsNotImplemented("DataBlobBase.set_validation()")
+
+    @property
+    def validation(self) -> Any:
+        """An object representing the validation set."""
+        if self._validation is None:
+            self._validation = self.set_validation()
+        return self._validation
+
+    def set_test(self) -> Any:
+        """Creates and returns a new object representing the test set."""
+        raise IsNotImplemented("DataBlobBase.set_test()")
+
+    @property
+    def test(self) -> Any:
+        """An object representing the test set."""
+        if self._test is None:
+            self._test = self.set_test()
+        return self._test
+
+
+class DataBlob(DataBlobBase):
     """
     Subclass this to group your training, validation, and test sets for training machine learning models.
 
@@ -201,31 +245,6 @@ class DataBlob(SingleNamespace):
     _training: Optional[tf.data.Dataset] = None
     _validation: Optional[tf.data.Dataset] = None
     _test: Optional[tf.data.Dataset] = None
-    _hyperparams_flat: Optional[Dict[str, Any]] = None
-
-    @property
-    def hyperparams_flat(self) -> Dict[str, Any]:
-        """
-        Returns a Python dictionary of "flattened" hyperparameters.
-
-        :py:class:`AppendDataBlob` objects modify a
-        "parent" :py:class:`DataBlob`, nesting the parent's
-        `Hyperparams` within the :py:class:`AppendDataBlob` 's
-        own `Hyperparams`.
-
-        This makes it hard to look up a given hyperparams
-        key. A value at ``parent_datablob.hyperparams.a`` is
-        stored at ``child_datablob.hyperparams.parent.hyperparams.a``.
-
-        This ``hyperparams_flat`` property provides all
-        nested hyperparams keys as a flat Python dictionary.
-        If a child :py:class:`AppendDataBlob` has a hyperparameter
-        key that that conflicts with the parent, the child's value
-        will overwrite the parent's value.
-        """
-        if self._hyperparams_flat is None:
-            self._hyperparams_flat = flatten_hyperparams(self.hyperparams)
-        return self._hyperparams_flat
 
     @classmethod
     def from_filesystem(
@@ -237,7 +256,7 @@ class DataBlob(SingleNamespace):
         shard_quantity: int = 1,
     ):
         """
-        Load a :py:class:`DataBlob` from the filesystem, calculating the
+        Loads a :py:class:`DataBlob` from the filesystem, calculating the
         filename from the hyperparameters.
 
         Args:
@@ -253,6 +272,74 @@ class DataBlob(SingleNamespace):
             path, shard_offset=shard_offset, shard_quantity=shard_quantity
         )
 
+    @classmethod
+    def from_filesystem_distributed(
+        cls,
+        *,
+        hyperparams: Optional[Union[Mapping[str, Any], HyperparamsType]] = None,
+        datablobs_directory: str,
+        repeat: Union[bool, int, None] = True,
+        per_replica_batch_size: Optional[int] = None,
+        tf_distribute_strategy: Optional[tf.distribute.Strategy] = None,
+    ) -> "DistributedDataBlob":
+        """
+        Loads a sharded :py:class:`DataBlob` from the filesystem,
+        automatically splitting the shards amongs the input workers
+        of a :py:class:`tf.distribute.Strategy`.
+
+        Args:
+            hyperparams: The hyperparameters of the model that we want to load.
+
+            datablobs_directory: The parent directory of all of your saved
+                :py:class:`DataBlob` s. The exact filename is calculated
+                from the class name and hyperparams.
+
+            repeat: Repeats the :py:class:`DataBlob` after loading it.
+                Set to ``True`` to enable infinite repeating.
+                Set to a positive integer ``n`` to repeat the
+                :py:class:`DataBlob` ``n`` times.
+                Set to ``False`` to disable repeating.
+
+            per_replica_batch_size: The batch size for each individual
+                :py:mod:`tf.distribute` replica. This is the global
+                batch size divided by :py:attr:`tf.distribute.Strategy.num_replicas_in_sync`.
+
+            tf_distribute_strategy: The :py:class:`tf.distribute.Strategy`
+                subclass to use. Optionally, this method will detect if it
+                is already inside a `:py:meth:`tf.distribute.Strategy.scope`
+                context manager.
+        """
+        return _DistributedDataBlobFromFilesystem(
+            hyperparams=hyperparams,
+            datablobs_directory=datablobs_directory,
+            datablob_class=cls,
+            repeat=repeat,
+            per_replica_batch_size=per_replica_batch_size,
+            tf_distribute_strategy=tf_distribute_strategy,
+        )
+
+    @classmethod
+    def metadata_from_filesystem(
+        cls,
+        *,
+        hyperparams: Optional[Union[Mapping[str, Any], HyperparamsType]] = None,
+        datablobs_directory: str,
+    ) -> DataBlobMetadata:
+        """
+        Loads this :py:class:`DataBlob` 's :py:class:`DataBlobMetadata`
+        from the filesystem, calculating the filename from the hyperparameters.
+
+        Args:
+            hyperparams: The hyperparameters of the model that we want to load.
+
+            datablobs_directory: The parent directory of all of your saved
+                :py:class:`DataBlob` s. The exact filename is calculated
+                from the class name and hyperparams.
+        """
+        name = cls.calculate_name(hyperparams=hyperparams)
+        path = os.path.join(datablobs_directory, name)
+        return cls.metadata_from_exact_path(path)
+
     @classmethod
     def from_filesystem_or_new(
         cls,
@@ -296,6 +383,51 @@ class DataBlob(SingleNamespace):
             path=path, shard_offset=shard_offset, shard_quantity=shard_quantity
         )
 
+    @classmethod
+    def from_exact_path_distributed(
+        cls,
+        *,
+        path: str,
+        repeat: Union[bool, int, None] = True,
+        per_replica_batch_size: Optional[int] = None,
+        tf_distribute_strategy: Optional[tf.distribute.get_strategy] = None,
+    ) -> "DistributedDataBlob":
+        """
+        Args:
+            path: The exact location of the saved :py:class:`DataBlob`
+                on the filesystem.
+
+            repeat: Repeats the :py:class:`DataBlob` after loading it.
+                Set to ``True`` to enable infinite repeating.
+                Set to a positive integer ``n`` to repeat the
+                :py:class:`DataBlob` ``n`` times.
+                Set to ``False`` to disable repeating.
+
+            per_replica_batch_size: The batch size for each individual
+                :py:mod:`tf.distribute` replica. This is the global
+                batch size divided by :py:attr:`tf.distribute.Strategy.num_replicas_in_sync`.
+
+            tf_distribute_strategy: The :py:class:`tf.distribute.Strategy`
+                subclass to use. Optionally, this method will detect if it
+                is already inside a `:py:meth:`tf.distribute.Strategy.scope`
+                context manager.
+        """
+        return _DistributedDataBlobFromExactPath(
+            path=path,
+            datablob_class=cls,
+            repeat=repeat,
+            per_replica_batch_size=per_replica_batch_size,
+            tf_distribute_strategy=tf_distribute_strategy,
+        )
+
+    @staticmethod
+    def metadata_from_exact_path(path: str) -> DataBlobMetadata:
+        """
+        Loads this :py:class:`DataBlob` 's :py:class:`DataBlobMetadata`
+        from a directory on the filesystem.
+        """
+        return DataBlobMetadata.load(path)
+
     def exists_in_datablobs_directory(
         self,
         datablobs_directory: str,
@@ -439,6 +571,76 @@ class DataBlob(SingleNamespace):
             precache_test=precache_test,
         )
 
+    def prefetch(
+        self,
+        buffer_size: int,
+        *,
+        training: bool = True,
+        validation: bool = True,
+        test: bool = True,
+    ) -> "DataBlob":
+        """
+        Creates a :py:class:`DataBlob` that prefetches elements for
+        performance.
+
+        Args:
+            buffer_size: The maximum number of elements that will
+                be buffered when prefetching. If the value
+                :py:meth:`tf.data.experimental.AUTOTUNE` is used,
+                then the buffer is dynamically tuned.
+
+            training: Apply the repeat operator to the training set.
+                Defaults to ``True``.
+
+            validation: Apply the repeat operator to the validation set.
+                Defaults to ``True``.
+
+            test: Apply the repeat operator to the test set.
+                Defaults to ``True``.
+        """
+        return _PrefetchDataBlob(
+            wraps=self,
+            buffer_size=buffer_size,
+            training=training,
+            validation=validation,
+            test=test,
+        )
+
+    def repeat(
+        self,
+        count: Optional[int] = None,
+        *,
+        training: bool = True,
+        validation: bool = True,
+        test: bool = True,
+    ) -> "DataBlob":
+        """
+        Repeats this :py:class:`DataBlob`.
+
+        Args:
+            count: Represents the number of times that the
+                elements in the :py:class:`tf.data.Dataset` should
+                be repeated. The default behavior (if ``count`` is
+                ``None`` or ``-1``) is for the dataset be repeated
+                indefinitely.
+
+            training: Apply the repeat operator to the training set.
+                Defaults to ``True``.
+
+            validation: Apply the repeat operator to the validation set.
+                Defaults to ``True``.
+
+            test: Apply the repeat operator to the test set.
+                Defaults to ``True``.
+        """
+        return _RepeatDataBlob(
+            wraps=self,
+            count=count,
+            training=training,
+            validation=validation,
+            test=test,
+        )
+
     def with_options(
         self,
         options: tf.data.Options,
@@ -512,48 +714,12 @@ class DataBlob(SingleNamespace):
         temp_datablob_path = temporary_filename(final_datablob_path)
         os.mkdir(temp_datablob_path)
         try:
-            # Save metadata as JSON so it is human-readable on the filesystem.
-            metadata_json_path = os.path.join(
-                temp_datablob_path, _METADATA_JSON_FILENAME
-            )
-
-            # These are the fields that we are going to identically save
-            # in both the JSON and the Pickle.
-            common_metadata = dict(
-                name=self.name,
-                group_name=self.group_name,
+            # Save DataBlob metadata to JSON and Pickle.
+            DataBlobMetadata.from_datablob(
+                datablob=self,
                 save_load_version=save_load_version,
                 num_shards=num_shards,
-            )
-
-            with open(metadata_json_path, "w", encoding="utf-8") as fh:
-                json.dump(
-                    obj=dict(
-                        **common_metadata,
-                        # When we save hyperparams to JSON, we first.
-                        # convert Python dataclasses to dictionaries.
-                        hyperparams=asdict(self.hyperparams),
-                    ),
-                    fp=fh,
-                    sort_keys=True,
-                    indent=4,
-                )
-            # Save the hyperparameter again in the pickle format.
-            # This is better for actually deserializing them.
-            metadata_pickle_path = os.path.join(
-                temp_datablob_path, _METADATA_PICKLE_FILENAME
-            )
-            with open(metadata_pickle_path, "wb") as fh:  # type: ignore
-                scalarstop.pickle.dump(
-                    obj=dict(
-                        **common_metadata,
-                        # When we save hyperparams to Pickle, we make
-                        # sure to serialize the hyperparams as a Python
-                        # dataclass.
-                        hyperparams=self.hyperparams,
-                    ),
-                    file=fh,
-                )
+            ).save(temp_datablob_path)
 
             for subtype in _SUBTYPES:
                 # Create the directory for each subtype.
@@ -823,7 +989,7 @@ class _WrapDataBlob(DataBlob):
     ):  # pylint: disable=super-init-not-called
         self._wraps = wraps
         self.Hyperparams = self._wraps.Hyperparams
-        self.hyperparams = self._wraps.hyperparams
+        self._hyperparams = self._wraps.hyperparams
         self._name = wraps.name
         self._group_name = wraps.group_name
         self._enable_training = training
@@ -1271,6 +1437,77 @@ class _CacheDataBlob(_WrapDataBlob):
         return tfdata.cache()
 
 
+class _PrefetchDataBlob(_WrapDataBlob):
+    """
+    Prefetch elements from this :py:class:`DataBlob` for improved latency.
+    """
+
+    def __init__(
+        self,
+        *,
+        wraps: Any,
+        buffer_size: int,
+        training: bool,
+        validation: bool,
+        test: bool,
+    ):
+        super().__init__(
+            wraps=wraps, training=training, validation=validation, test=test
+        )
+        self._buffer_size = buffer_size
+
+    def _wrap_tfdata(self, tfdata: tf.data.Dataset) -> tf.data.Dataset:
+        return tfdata.prefetch(self._buffer_size)
+
+
+class _RepeatDataBlob(_WrapDataBlob):
+    """
+    Repeats this :py:class:`DataBlob` a given number of times.
+    """
+
+    def __init__(
+        self,
+        *,
+        wraps: Any,
+        count: Optional[int] = None,
+        training: bool,
+        validation: bool,
+        test: bool,
+    ):
+        super().__init__(
+            wraps=wraps, training=training, validation=validation, test=test
+        )
+        self._count = count
+
+    def _wrap_tfdata(self, tfdata: tf.data.Dataset) -> tf.data.Dataset:
+        return tfdata.repeat(self._count)
+
+
+class _ShardDataBlob(_WrapDataBlob):
+    """
+    Shards this :py:class:`DataBlob`.
+    """
+
+    def __init__(
+        self,
+        *,
+        wraps: Any,
+        num_shards: int,
+        shard_index: int,
+        training: bool,
+        validation: bool,
+        test: bool,
+    ):
+        super().__init__(
+            wraps=wraps, training=training, validation=validation, test=test
+        )
+        self._num_shards = num_shards
+        self._shard_index = shard_index
+
+    def _wrap_tfdata(self, tfdata: tf.data.Dataset) -> tf.data.Dataset:
+        return tfdata.shard(self._num_shards, self._shard_index)
+
+
 class _WithOptionsDataBlob(_WrapDataBlob):
     """
     Apply a :py:class:`tf.data.Options` object to this :py:class:`DataBlob`.
@@ -1328,28 +1565,42 @@ class _LoadDataBlob(DataBlob):
         shard_offset: Optional[int] = None,
         shard_quantity: int = 1,
     ) -> Union[DataBlob, DataFrameDataBlob]:
-        metadata_path = os.path.join(path, _METADATA_PICKLE_FILENAME)
-        try:
-            with open(metadata_path, "rb") as fh:
-                metadata = scalarstop.pickle.load(fh)
-        except FileNotFoundError as exc:
-            raise DataBlobNotFound(path) from exc
-        name = metadata["name"]
-        group_name = metadata["group_name"]
-        save_load_version = metadata.get("save_load_version", 1)
-        total_num_shards = metadata.get("num_shards", 1)
-        hyperparams = metadata["hyperparams"]
+        metadata = DataBlobMetadata.load(path)
         return cls(
             path=path,
-            name=name,
-            group_name=group_name,
-            hyperparams=hyperparams,
-            total_num_shards=total_num_shards,
+            name=metadata.name,
+            group_name=metadata.group_name,
+            hyperparams=metadata.hyperparams,
+            total_num_shards=metadata.num_shards,
+            save_load_version=metadata.save_load_version,
             shard_offset=shard_offset,
             shard_quantity=shard_quantity,
-            save_load_version=save_load_version,
         )
 
+    @property
+    def total_num_shards(self) -> int:
+        """
+        The total number of shards for this :py:class:`DataBlob` that were
+        saved to the filesystem.
+        """
+        return self._total_num_shards
+
+    @property
+    def shard_offset(self) -> Optional[int]:
+        """
+        The start offset for the range of shards that this :py:class:`DataBlob`
+        is loading from the filesystem.
+        """
+        return self._shard_offset
+
+    @property
+    def shard_quantity(self) -> int:
+        """
+        The number of shards that this :py:class:`DataBlob` is loading
+        from the filesystem.
+        """
+        return self._shard_quantity
+
     def _load_tfdata(self, subtype: str) -> tf.data.Dataset:
         """Load one of the :py:class:`tf.data.Dataset` s that we have saved."""
         try:
@@ -1399,3 +1650,312 @@ class _LoadDataFrameDataBlob(_LoadDataBlob, DataFrameDataBlob):
 
     def set_test_dataframe(self) -> pd.DataFrame:
         return self._set_subtype_dataframe(_SUBTYPE_TEST)
+
+
+class DistributedDataBlob(DataBlobBase):
+    """
+    Wraps a :py:class:`DataBlob` to create a TensorFlow :py:class:`tf.distribute.DistributedDataset`.
+
+    A :py:class:`DataBlob` contains three TensorFlow :py:class:`tf.data.Dataset`
+    pipelines, representing a training, validation, and test set.
+    The :py:class:`DistributedDataBlob`  wraps the creation of a :py:class:`DataBlob`
+    to turn each :py:class:`tf.data.Dataset` into a :py:class:`tf.distribute.DistributedDataset`
+    which is used to distribute a dataset across multiple workers according
+    to a :py:class:`tf.distribute.Strategy`.
+
+    If you have saved a :py:class:`DataBlob` to the filesystem with
+    :py:meth:`DataBlob.save`, then you can automatically load the :py:class:`DataBlob`
+    from the filesystem as a :py:class:`DistributedDataBlob` using the
+    classmethod :py:meth:`DataBlob.from_filesystem_distributed` or
+    :py:meth:`DataBlob.from_exact_path_distributed`.
+
+    For more fine-grained control, you can subclass :py:class:`DistributedDataBlob`
+    and override :py:meth:`DistributedDataBlob.new_sharded_datablob` with
+    your own :py:class:`DataBlob` creation and sharding logic. Optionally,
+    you can also subclass :py:meth:`DistributedDataBlob.transform_datablob`
+    to change how :py:class:`DistributedDataBlob` handles repeating and batching.
+    Finally, you can also subclass :py:meth:`DistributedDataBlob.postprocess_tfdata`
+    to make changes to individual :py:class:`tf.data.Dataset` instances rather
+    than the :py:class:`DataBlob` as a whole.
+    """  # pylint: disable=line-too-long
+
+    def __init__(
+        self,
+        *,
+        name: str,
+        group_name: str,
+        hyperparams: Optional[Union[Mapping[str, Any], HyperparamsType]] = None,
+        hyperparams_class: Type[HyperparamsType],
+        repeat: Union[bool, int, None] = True,
+        per_replica_batch_size: Optional[int] = None,
+        tf_distribute_strategy: Optional[tf.distribute.get_strategy] = None,
+    ):
+        """
+        Args:
+            name: The name of the wrapped :py:class:`DataBlob`.
+
+            group_name: The group name of the wrapped :py:class:`DataBlob`.
+
+            hyperparams: The hyperparameters of the wrapped :py:class:`DataBlob`.
+
+            hyperparams_class: The :py:class:`HyperparamsType` class that
+                ``hyperparams`` instances are created from.
+
+            repeat: Repeats the :py:class:`DataBlob` after loading it.
+                Set to ``True`` to enable infinite repeating.
+                Set to a positive integer ``n`` to repeat the
+                :py:class:`DataBlob` ``n`` times.
+                Set to ``False`` to disable repeating.
+
+            per_replica_batch_size: The batch size for each individual
+                :py:mod:`tf.distribute` replica. This is the global
+                batch size divided by :py:attr:`tf.distribute.Strategy.num_replicas_in_sync`.
+
+            tf_distribute_strategy: The :py:class:`tf.distribute.Strategy`
+                subclass to use. Optionally, this method will detect if it
+                is already inside a `:py:meth:`tf.distribute.Strategy.scope`
+                context manager.
+        """
+        self._name = name
+        self._group_name = group_name
+        self.Hyperparams = hyperparams_class
+        super().__init__(
+            hyperparams=hyperparams,
+        )
+        self._repeat = repeat
+        self._per_replica_batch_size = per_replica_batch_size
+        self._tf_distribute_strategy = (
+            tf_distribute_strategy or tf.distribute.get_strategy()
+        )
+        self._training: Optional[tf.data.Dataset] = None
+        self._validation: Optional[tf.data.Dataset] = None
+        self._test: Optional[tf.data.Dataset] = None
+
+    def __repr__(self) -> str:
+        return f"<sp.DistributedDataBlob {self.name}>"
+
+    def new_sharded_datablob(
+        self, ctx: tf.distribute.InputContext  # pylint: disable=unused-argument
+    ) -> DataBlob:
+        """
+        Subclass this method to return a sharded :py:class:`DataBlob`.
+
+        Args:
+            ctx: A :py:class:`tf.distribute.InputContext` instance.
+                The attribute :py:attr:`tf.distribute.InputContext.input_pipeline_id`
+                returns the current input pipeline.
+                The attribute :py:attr:`tf.distribute.InputContext.num_input_pipelines`
+                returns the total number of distributed input pipelines
+                in the current :py:class:`tf.distribute.Strategy`.
+
+        """
+        raise NotImplementedError(
+            "Subclass DistributedDataBlob.new_sharded_datablob() to return a sharded DataBlob."
+        )
+
+    def transform_datablob(  # pylint: disable=unused-argument
+        self,
+        datablob: DataBlob,
+        ctx: tf.distribute.InputContext,
+    ) -> DataBlob:
+        """
+        Transforms an already-initialized :py:class:`DataBlob` to add repeating and sharding logic.
+
+        Args:
+            datablob: The already-initialized :py:class:`DataBlob`.
+
+            ctx: A :py:class:`tf.distribute.InputContext` instance.
+                The attribute :py:attr:`tf.distribute.InputContext.input_pipeline_id`
+                returns the current input pipeline.
+                The attribute :py:attr:`tf.distribute.InputContext.num_input_pipelines`
+                returns the total number of distributed input pipelines
+                in the current :py:class:`tf.distribute.Strategy`.
+
+        Returns:
+            Returns a :py:class:`DataBlob` that has been modified by
+                repeating, batching, or another transformation.
+        """
+        if self._repeat is True:
+            datablob = datablob.repeat()
+        elif self._repeat is not False:
+            datablob = datablob.repeat(count=self._repeat)
+        if self._per_replica_batch_size is not None:
+            datablob = datablob.batch(
+                batch_size=self._per_replica_batch_size,
+            )
+        return datablob
+
+    def postprocess_tfdata(  # pylint: disable=unused-argument
+        self,
+        tfdata: tf.data.Dataset,
+        ctx: tf.distribute.InputContext,
+    ) -> tf.data.Dataset:
+        """
+        Performs additional :py:class:`tf.data.Dataset` transformations
+        before turning them into :py:class:`tf.distribute.DistributedDataset`
+        instances.
+
+        Currently, the implementation in :py:class:`DistributedDataBlob`
+        does nothing, but is avaiable for you to subclass and change.
+
+        Args:
+            tfdata: The input :py:class:`tf.data.Dataset` instance to transform.
+
+            ctx: A :py:class:`tf.distribute.InputContext` instance.
+                The attribute :py:attr:`tf.distribute.InputContext.input_pipeline_id`
+                returns the current input pipeline.
+                The attribute :py:attr:`tf.distribute.InputContext.num_input_pipelines`
+                returns the total number of distributed input pipelines
+                in the current :py:class:`tf.distribute.Strategy`.
+
+        Returns:
+            Returns a transformed :py:class:`tf.data.Dataset`.
+        """
+        return tfdata
+
+    def _get_distributed_tfdata(self, subtype: str) -> tf.distribute.DistributedDataset:
+        """
+        Returns a :py:class:`tf.distribute.DistributedDataset` for the
+        training, validation, or test set.
+
+        Args:
+            subtype: Either ``"training"``, ``"validation"``, or ``"test``.
+        """
+
+        def _inner(ctx: tf.distribute.InputContext):
+            datablob = self.new_sharded_datablob(ctx)
+            datablob = self.transform_datablob(datablob, ctx)
+            if subtype == _SUBTYPE_TRAINING:
+                tfdata = datablob.set_training()
+            elif subtype == _SUBTYPE_VALIDATION:
+                tfdata = datablob.set_validation()
+            elif subtype == _SUBTYPE_TEST:
+                tfdata = datablob.set_test()
+            else:
+                raise RuntimeError(f"Invalid {subtype=}.")
+            return self.postprocess_tfdata(ctx=ctx, tfdata=tfdata)
+
+        return self._tf_distribute_strategy.distribute_datasets_from_function(_inner)
+
+    @property
+    def tf_distribute_strategy(self) -> tf.distribute.Strategy:
+        """Returns the currently-active :py:class:`tf.distribute.Strategy`."""
+        return self._tf_distribute_strategy
+
+    def set_training(self) -> tf.distribute.DistributedDataset:
+        """Creates a new :py:class:`tf.distribute.DistributedDataset` for the training set."""
+        return self._get_distributed_tfdata(_SUBTYPE_TRAINING)
+
+    @property
+    def training(self) -> tf.distribute.DistributedDataset:
+        """A :py:class:`tf.distribute.DistributedDataset` instance for the training set."""
+        if self._training is None:
+            self._training = self.set_training()
+        return self._training
+
+    def set_validation(self) -> tf.distribute.DistributedDataset:
+        """Creates a new :py:class:`tf.distribute.DistributedDataset` for the validation set."""
+        return self._get_distributed_tfdata(_SUBTYPE_VALIDATION)
+
+    @property
+    def validation(self) -> tf.distribute.DistributedDataset:
+        """A :py:class:`tf.distribute.DistributedDataset` instance for the validation set."""
+        if self._validation is None:
+            self._validation = self.set_validation()
+        return self._validation
+
+    def set_test(self) -> tf.distribute.DistributedDataset:
+        """Creates a new :py:class:`tf.distribute.DistributedDataset` for the test set."""
+        return self._get_distributed_tfdata(_SUBTYPE_TEST)
+
+    @property
+    def test(self) -> tf.distribute.DistributedDataset:
+        """A :py:class:`tf.distribute.DistributedDataset` instance for the test set."""
+        if self._test is None:
+            self._test = self.set_test()
+        return self._test
+
+
+class _DistributedDataBlobFromFilesystem(DistributedDataBlob):
+    """Implements :py:meth:`DataBlob.from_filesystem_distributed`."""
+
+    def __init__(
+        self,
+        *,
+        hyperparams: Optional[Union[Mapping[str, Any], HyperparamsType]] = None,
+        datablobs_directory: str,
+        datablob_class: Optional[Type[DataBlob]] = None,
+        repeat: Union[bool, int, None] = True,
+        per_replica_batch_size: Optional[int] = None,
+        tf_distribute_strategy: Optional[tf.distribute.get_strategy] = None,
+    ):
+        if datablob_class is None:
+            self._datablob_class = DataBlob
+        else:
+            self._datablob_class = datablob_class
+        metadata = self._datablob_class.metadata_from_filesystem(
+            hyperparams=hyperparams,
+            datablobs_directory=datablobs_directory,
+        )
+        super().__init__(
+            name=metadata.name,
+            group_name=metadata.group_name,
+            hyperparams=metadata.hyperparams,
+            hyperparams_class=metadata.hyperparams.__class__,
+            repeat=repeat,
+            per_replica_batch_size=per_replica_batch_size,
+            tf_distribute_strategy=tf_distribute_strategy,
+        )
+        self._datablobs_directory = datablobs_directory
+        self._num_shards = metadata.num_shards
+
+    def new_sharded_datablob(self, ctx: tf.distribute.InputContext):
+        shard_quantity = self._num_shards // ctx.num_input_pipelines
+        shard_offset = shard_quantity * ctx.input_pipeline_id
+        return self._datablob_class.from_filesystem(
+            hyperparams=self._hyperparams,
+            datablobs_directory=self._datablobs_directory,
+            shard_offset=shard_offset,
+            shard_quantity=shard_quantity,
+        )
+
+
+class _DistributedDataBlobFromExactPath(DistributedDataBlob):
+    """Implements :py:meth:`DataBlob.from_exact_path_distributed`."""
+
+    def __init__(
+        self,
+        *,
+        path: str,
+        datablob_class: Optional[Type[DataBlob]] = None,
+        repeat: Union[bool, int, None] = True,
+        per_replica_batch_size: Optional[int] = None,
+        tf_distribute_strategy: Optional[tf.distribute.get_strategy] = None,
+    ):
+        if datablob_class is None:
+            self._datablob_class = DataBlob
+        else:
+            self._datablob_class = datablob_class
+        metadata = self._datablob_class.metadata_from_exact_path(
+            path=path,
+        )
+        super().__init__(
+            name=metadata.name,
+            group_name=metadata.group_name,
+            hyperparams=metadata.hyperparams,
+            hyperparams_class=metadata.hyperparams.__class__,
+            repeat=repeat,
+            per_replica_batch_size=per_replica_batch_size,
+            tf_distribute_strategy=tf_distribute_strategy,
+        )
+        self._path = path
+        self._num_shards = metadata.num_shards
+
+    def new_sharded_datablob(self, ctx: tf.distribute.InputContext):
+        shard_quantity = self._num_shards // ctx.num_input_pipelines
+        shard_offset = shard_quantity * ctx.input_pipeline_id
+        return DataBlob.from_exact_path(
+            path=self._path,
+            shard_offset=shard_offset,
+            shard_quantity=shard_quantity,
+        )
