@@ -145,7 +145,7 @@ from log_with_context import Logger
 
 from scalarstop._filesystem import rmtree
 from scalarstop._keras_callbacks import BatchLoggingCallback, EpochCallback
-from scalarstop.datablob import DataBlob
+from scalarstop.datablob import DataBlob, DistributedDataBlob
 from scalarstop.exceptions import IsNotImplemented, ModelNotFoundError
 from scalarstop.model_template import ModelTemplate
 from scalarstop.train_store import TrainStore
@@ -163,16 +163,21 @@ class Model:
 
     @classmethod
     def from_filesystem(
-        cls, *, datablob: DataBlob, model_template: ModelTemplate, models_directory: str
+        cls,
+        *,
+        datablob: Union[DataBlob, DistributedDataBlob],
+        model_template: ModelTemplate,
+        models_directory: str,
     ) -> "Model":
         """
         Load an already-trained model from the filesystem.
 
         Args:
-            datablob: The :py:class:`~scalarstop.models.DataBlob`
+            datablob: The :py:class:`~scalarstop.datablob.DataBlob`
+                or :py:class:`~scalarstop.datablob.DistributedDataBlob`
                 used to train the model that we are looking for.
 
-            model_template: The :py:class:`~scalarstop.ModelTemplate`
+            model_template: The :py:class:`~scalarstop.model_template.ModelTemplate`
                 used to create the model that we are looking for.
 
             models_directory: The directory where you store all of your
@@ -193,17 +198,22 @@ class Model:
 
     @classmethod
     def from_filesystem_or_new(
-        cls, *, datablob: DataBlob, model_template: ModelTemplate, models_directory: str
+        cls,
+        *,
+        datablob: Union[DataBlob, DistributedDataBlob],
+        model_template: ModelTemplate,
+        models_directory: str,
     ) -> "Model":
         """
         Load a saved model from the filesystem. If we can't find one, create a new one with
-        the supplied :py:class:`~scalarstop.ModelTemplate`.
+        the supplied :py:class:`~scalarstop.model_template.ModelTemplate`.
 
         Args:
-            datablob: The :py:class:`~scalarstop.models.DataBlob`
+            datablob: The :py:class:`~scalarstop.datablob.DataBlob`
+                or :py:class:`~scalarstop.datablob.DistributedDataBlob`
                 that we will use to train the model.
 
-            model_template: The :py:class:`~scalarstop.ModelTemplate`
+            model_template: The :py:class:`~scalarstop.model_template.ModelTemplate`
                 that we will use to create the model.
 
             models_directory: The directory where you store all of your
@@ -225,7 +235,7 @@ class Model:
     def __init__(
         self,
         *,
-        datablob: DataBlob,
+        datablob: Union[DataBlob, DistributedDataBlob],
         model_template: ModelTemplate,
         model: Optional[Any] = None,
     ):
@@ -244,8 +254,8 @@ class Model:
     def calculate_name(*, model_template_name: str, datablob_name: str) -> str:
         """
         Create a model name from a
-        :py:class:`~scalarstop.ModelTemplate`
-        name and a :py:class:`~scalarstop.DataBlob` name.
+        :py:class:`~scalarstop.model_template.ModelTemplate`
+        name and a :py:class:`~scalarstop.datablob.DataBlob` name.
         """
         return f"mt_{model_template_name}__d_{datablob_name}"
 
@@ -256,16 +266,17 @@ class Model:
 
         If you intend on overriding this method, you should make sure
         that two :py:class:`Model` s trained on the same
-        :py:class:`~scalarstop.DataBlob` and
-        :py:class:`~scalarstop.ModelTemplate` have the
+        :py:class:`~scalarstop.datablob.DataBlob` and
+        :py:class:`~scalarstop.model_template.ModelTemplate` have the
         same name.
         """
         return self._name
 
     @property
-    def datablob(self) -> DataBlob:
+    def datablob(self) -> Union[DataBlob, DistributedDataBlob]:
         """
-        Returns the :py:class:`~scalarstop.DataBlob`
+        Returns the :py:class:`~scalarstop.datablob.DataBlob` or the
+        :py:class:`~scalarstop.datablob.DistributedDataBlob`
         used to create this model.
         """
         return self._datablob
@@ -273,7 +284,7 @@ class Model:
     @property
     def model_template(self) -> ModelTemplate:
         """
-        Returns the :py:class:`~scalarstop.ModelTemplate`
+        Returns the :py:class:`~scalarstop.model_template.ModelTemplate`
         used to create this model.
         """
         return self._model_template
@@ -315,7 +326,7 @@ class Model:
     def fit(self, *, final_epoch: int, **kwargs) -> Mapping[str, Sequence[float]]:
         """
         Fits the given model to the given
-        :py:class:`~scalarstop.DataBlob`.
+        :py:class:`~scalarstop.datablob.DataBlob`.
         """
         raise IsNotImplemented(f"{self.__class__.__name__}.fit()")
 
@@ -338,7 +349,7 @@ class KerasModel(Model):
     def from_filesystem(
         cls,
         *,
-        datablob: DataBlob,
+        datablob: Union[DataBlob, DistributedDataBlob],
         model_template: ModelTemplate,
         models_directory: str,
     ) -> "KerasModel":
@@ -376,7 +387,7 @@ class KerasModel(Model):
     def __init__(
         self,
         *,
-        datablob: DataBlob,
+        datablob: Union[DataBlob, DistributedDataBlob],
         model_template: ModelTemplate,
         model: Optional[Any] = None,
         history: Optional[_KERAS_HISTORY_TYPE] = None,
@@ -391,7 +402,16 @@ class KerasModel(Model):
             self._model.input_shape
         except AttributeError:
             x_spec, _ = self._datablob.training.element_spec
-            self._model.build(input_shape=x_spec.shape)
+            try:
+                x_spec_shape = x_spec.shape
+            except AttributeError:
+                _LOGGER.warning(
+                    "sp.KerasModel could not automatically determine "
+                    "input shape and has not called build() on the "
+                    "underlying Keras model object."
+                )
+            else:
+                self._model.build(input_shape=x_spec_shape)
 
     def __repr__(self) -> str:
         if self.current_epoch == 1:
@@ -448,11 +468,13 @@ class KerasModel(Model):
         train_store: Optional[TrainStore] = None,
         tensorboard_directory: Optional[str] = None,
         profile_batch: Union[int, Tuple[int, int]] = 0,
+        steps_per_epoch: Optional[int] = None,
+        validation_steps_per_epoch: Optional[int] = None,
         callbacks: Optional[Sequence[tf.keras.callbacks.Callback]] = None,
         **kwargs,
     ) -> Mapping[str, Sequence[float]]:
         """
-        Fit the Keras model to the :py:class:`~scalarstop.DataBlob`
+        Fit the Keras model to the :py:class:`~scalarstop.datablob.DataBlob`
         that this model was created for.
 
         Args:
@@ -477,8 +499,8 @@ class KerasModel(Model):
 
             train_store: A :py:class:`~scalarstop.TrainStore`
                 instance, which is a client that persists metadata about
-                :py:class:`~scalarstop.DataBlob` s,
-                :py:class:`~scalarstop.ModelTemplate` s,
+                :py:class:`~scalarstop.datablob.DataBlob` s,
+                :py:class:`~scalarstop.model_template.ModelTemplate` s,
                 and :py:class:`~scalarstop.model.Model` s.
 
             tensorboard_directory: A directory on the filesystem to write
@@ -489,6 +511,26 @@ class KerasModel(Model):
                 a valid filesystem path is given as
                 ``tensorboard_directory``.
 
+            steps_per_epoch: The total number of steps (batches of samples)
+                before declaring one training epoch as "finished" and starting
+                the next epoch. When ``steps_per_epoch`` is ``None``,
+                the epoch will run until the input
+                :py:attr:`DataBlob.training` is exhausted. When passing
+                an infinitely repeating dataset, you must specify
+                ``steps_per_epoch``. If ``steps_per_epoch = -1``, the
+                training will run indefinitely with an infinitely
+                repeating dataset.
+
+            validation_steps_per_epoch: The total number of steps
+                (batches of samples) before declaring one validation
+                epoch as "finished" and starting the next epoch.
+                If ``validation_steps_per_epoch`` is specified
+                and only part of :py:class:`DataBlob.validation`
+                is consumed, the evaluation of :py:class:`DataBlob.validation`
+                will start from the beginning of :py:class:`DataBlob.validation`
+                at every epoch. This ensures that the same validation
+                samples are used every time.
+
             callbacks: A list of Keras callbacks to use while training.
         """
         if kwargs:
@@ -559,6 +601,8 @@ class KerasModel(Model):
                 callbacks=callbacks,
                 initial_epoch=self.current_epoch,
                 epochs=final_epoch,
+                steps_per_epoch=steps_per_epoch,
+                validation_steps=validation_steps_per_epoch,
             )
         return self.history
 
@@ -597,14 +641,14 @@ class KerasModel(Model):
         callbacks: Optional[Sequence[tf.keras.callbacks.Callback]] = None,
     ) -> Sequence[float]:
         """
-        Evaluate this model on the :py:class:`~scalarstop.DataBlob`'s test set.
+        Evaluate this model on the :py:class:`~scalarstop.datablob.DataBlob`'s test set.
 
         Optionally, you can provide another :py:class:`tf.data.Dataset` via the
         ``dataset`` parameter.
 
         Args:
             dataset: Another :py:class:`tf.data.Dataset` to evalaute instead of
-                the test set of the provided :py:class:`~scalarstop.DataBlob`.
+                the test set of the provided :py:class:`~scalarstop.datablob.DataBlob`.
 
             verbose: Specifiy verbosity for evaluating this model.
 
